// Point&Figure create PNF array module
// Copyright (c) 2012-2014 Roman Rich
#include            "main.mq4"

// =================
// BOX_SIZE                                
// REVERSE_SIZE                           
// =================
// struct PNF_CHART                       
//  uint column_num                      
//  datetime last_date_time             /    
//  uchar column_type                    :   
//  double min_column_price                
//  double max_column_price                
//  ulong column_volume                   
//  double supp_price                      
//  double resist_price                    
// =================
// PNF_CHART pnf[]                        
// =================
// int column_count                       
// double double_box                       
// double start_min                            
// double start_max                            
// double cur_min                             
// double cur_max                             
// double rev_min                           ,      
// double rev_max                           ,      
// double cont_min                          ,     
// double cont_max                          ,     
// ulong cur_vol                          
// uchar cur_column                      :   
// uchar trend                            : U  D
// double temp_minimum_price[]                
// double temp_maximum_price[]                
// int order_col_number=0;               ,    

enum                BOX_SIZE{b_1=1,b_2=2,b_3=3,b_4=4,b_5=5,b_10=10,b_20=20,b_50=50,b_100=100,b_200=200,b_500=500};
enum                REVERSE_SIZE{one=1,two,three,four,five};

sinput              string              delimiter_3="";                         // === ===
input               BOX_SIZE            box=b_100;                              //     (2  4  )
input               REVERSE_SIZE        reverse=three;                          // , 
sinput              string              delimiter_4="";                         // === ===

struct PNF_CHART
{
	uint column_num;
	datetime last_date_time;
	uchar column_type;
	double min_column_price;
	double max_column_price;
	ulong column_volume;
	double supp_price;
	double resist_price;
};

PNF_CHART pnf[];

int column_count;
double double_box;
double start_min;
double start_max;
double cur_min;
double cur_max;
double rev_min;
double rev_max;
double cont_min;
double cont_max;
ulong cur_vol;
uchar cur_column;
uchar trend;
double temp_minimum_price[];
double temp_maximum_price[];
int order_col_number=0;

int CalcIndicators()
{
	CreatePNFArray();
	return(0);
};

int RecalcIndicators()
{
//        ,...
	CopyTime(symbol,time_frame,1,1,date_time);
	CopyHigh(symbol,time_frame,1,1,price_high);
	CopyLow(symbol,time_frame,1,1,price_low);
	CopyRealVolume(symbol,time_frame,1,1,volumes);
// 
	price_low[0]=NormalizeDouble(MathLog10(price_low[0]),symbol_digit);
	price_high[0]=NormalizeDouble(MathLog10(price_high[0]),symbol_digit);
	cur_min=MathNorm(price_low[0],double_box,true,symbol_digit);
	cur_max=MathNorm(price_high[0],double_box,false,symbol_digit);
	cur_vol=volumes[0];
// ...  PNF,...
	UpdatePNF();
// ...   PNF   -   ,...
	if(ArraySize(temp_minimum_price)!=column_count)
	{
		ArrayResize(temp_minimum_price,column_count,0);
		ArrayResize(temp_maximum_price,column_count,0);
		pnf[column_count-1].supp_price=-10.0;
		pnf[column_count-1].resist_price=-10.0;
	};
// ...  
	temp_minimum_price[column_count-1]=pnf[column_count-1].min_column_price;
	temp_maximum_price[column_count-1]=pnf[column_count-1].max_column_price;
	UpdateTrend();
	return(0);
};

int CreatePNFArray()
{
	column_count=1;
	bars_count=MathMin(100000,Bars(symbol,time_frame));
	cur_column='.';
	trend='.';
//   
	ArrayFree(pnf);
	ArrayFree(date_time);
	ArrayFree(price_high);
	ArrayFree(price_low);
	ArrayFree(volumes);
	ArrayFree(temp_minimum_price);
	ArrayFree(temp_maximum_price);
	ArrayResize(pnf,bars_count,0);
	ArrayResize(date_time,bars_count,0);
	ArrayResize(price_high,bars_count,0);
	ArrayResize(price_low,bars_count,0);
	ArrayResize(volumes,bars_count,0);
//   
	CopyTime(symbol,time_frame,0,bars_count,date_time);
	CopyHigh(symbol,time_frame,0,bars_count,price_high);
	CopyLow(symbol,time_frame,0,bars_count,price_low);
	CopyRealVolume(symbol,time_frame,0,bars_count,volumes);
// 
	for(count_1=0;count_1<bars_count;count_1++)
	{
		price_low[count_1]=NormalizeDouble(MathLog10(price_low[count_1]),symbol_digit);
		price_high[count_1]=NormalizeDouble(MathLog10(price_high[count_1]),symbol_digit);
	};
//   ...
	double_box=NormalizeDouble((double)box/MathPow(10.0,(double)symbol_digit),symbol_digit);
	start_min=MathNorm(price_low[0],double_box,true,symbol_digit);
	start_max=MathNorm(price_high[0],double_box,false,symbol_digit);
	cont_min=NormalizeDouble(start_min-double_box,symbol_digit);
	cont_max=NormalizeDouble(start_max+double_box,symbol_digit);
	rev_min=NormalizeDouble(start_max-reverse*double_box,symbol_digit);
	rev_max=NormalizeDouble(start_min+reverse*double_box,symbol_digit);
	cur_vol=volumes[0];
// ...      
	pnf[column_count-1].column_num=column_count;
	pnf[column_count-1].column_type=cur_column;
	pnf[column_count-1].min_column_price=start_min;
	pnf[column_count-1].max_column_price=start_max;
	pnf[column_count-1].column_volume=cur_vol;
//    -
	for(count_1=1;count_1<bars_count;count_1++)
	{
		cur_min=MathNorm(price_low[count_1],double_box,true,symbol_digit);
		cur_max=MathNorm(price_high[count_1],double_box,false,symbol_digit);
		cur_vol=volumes[count_1];
		UpdatePNF();
	};
//   
//     ...
	ArrayResize(temp_minimum_price,column_count,0);
	ArrayResize(temp_maximum_price,column_count,0);
	for(count_2=0;count_2<column_count;count_2++)
	{
		pnf[count_2].supp_price=-10.0;
		pnf[count_2].resist_price=-10.0;
		temp_minimum_price[count_2]=pnf[count_2].min_column_price;
		temp_maximum_price[count_2]=pnf[count_2].max_column_price;
	}
// ...       (   )
	if(trend=='U')
	{
		pnf[0].supp_price=NormalizeDouble(temp_minimum_price[0]-double_box,symbol_digit);
		pnf[0].resist_price=-10.0;
	}
	else
	{
		pnf[0].supp_price=-10.0;
		pnf[0].resist_price=NormalizeDouble(temp_maximum_price[0]+double_box,symbol_digit);
	}
	count_2=count_3=1;
	while(count_2<column_count)
		CalcTrend();
//         1
	ArrayFree(date_time);
	ArrayFree(price_high);
	ArrayFree(price_low);
	ArrayFree(volumes);
	ArrayResize(date_time,1,0);
	ArrayResize(price_high,1,0);
	ArrayResize(price_low,1,0);
	ArrayResize(volumes,1,0);
	return(0);
};

int UpdatePNF()
{
	switch(cur_column)
	{
//      ,    :   
		case '.':
		{
//        ,      Down
			if(cur_max>=rev_max)
			{
				cur_column='X';
				cont_max=NormalizeDouble(cur_max+double_box,symbol_digit);
				rev_min=NormalizeDouble(cur_max-reverse*double_box,symbol_digit);
				trend='D';
				pnf[column_count-1].column_volume=pnf[column_count-1].column_volume+cur_vol;
				pnf[column_count-1].column_type=cur_column;
				pnf[column_count-1].max_column_price=cur_max;
				break;
			};
//         ,      Up
			if(cur_min<=rev_min)
			{
				cur_column='O';
				cont_min=NormalizeDouble(cur_min-double_box,symbol_digit);
				rev_max=NormalizeDouble(cur_min+reverse*double_box,symbol_digit);
				trend='U';
				pnf[column_count-1].column_volume=pnf[column_count-1].column_volume+cur_vol;
				pnf[column_count-1].column_type=cur_column;
				pnf[column_count-1].min_column_price=cur_min;
				break;
			};
			break;
		};
//  ,   - -...
		case 'X':
		{
// ...      ,...
			if(cur_max>=cont_max)
			{
				cont_max=NormalizeDouble(cur_max+double_box,symbol_digit);
				rev_min=NormalizeDouble(cur_max-reverse*double_box,symbol_digit);
				pnf[column_count-1].column_volume=pnf[column_count-1].column_volume+cur_vol;
				pnf[column_count-1].max_column_price=cur_max;
				break;
			};
// ...   -    -
			if(cur_min<=rev_min)
			{
				cur_column='O';
				column_count++;
				cont_min=NormalizeDouble(cur_min-double_box,symbol_digit);
				rev_max=NormalizeDouble(cur_min+reverse*double_box,symbol_digit);
				pnf[column_count-1].column_num=column_count;
				pnf[column_count-1].column_volume=cur_vol;
				pnf[column_count-1].column_type=cur_column;
				pnf[column_count-1].min_column_price=cur_min;
				pnf[column_count-1].max_column_price=NormalizeDouble(pnf[column_count-2].max_column_price-double_box,symbol_digit);
				break;
			};
			break;
		};
//   ,   - -...
		case 'O':
		{
// ...      ,...
			if(cur_min<=cont_min)
			{
				cont_min=NormalizeDouble(cur_min-double_box,symbol_digit);
				rev_max=NormalizeDouble(cur_min+reverse*double_box,symbol_digit);
				pnf[column_count-1].column_volume=pnf[column_count-1].column_volume+cur_vol;
				pnf[column_count-1].min_column_price=cur_min;
				break;
			};
// ...   -    -
			if(cur_max>=rev_max)
			{
				cur_column='X';
				column_count++;
				cont_max=NormalizeDouble(cur_max+double_box,symbol_digit);
				rev_min=NormalizeDouble(cur_max-reverse*double_box,symbol_digit);
				pnf[column_count-1].column_num=column_count;
				pnf[column_count-1].column_volume=cur_vol;
				pnf[column_count-1].column_type=cur_column;
				pnf[column_count-1].min_column_price=NormalizeDouble(pnf[column_count-2].min_column_price+double_box,symbol_digit);
				pnf[column_count-1].max_column_price=cur_max;
				break;
			};
			break;
		};
	};
	return(0);
};

int CalcTrend()
{
	switch(trend)
	{
//   - Up...
		case 'U':
		{
			temp_1=NormalizeDouble(pnf[count_2-1].supp_price+double_box,symbol_digit);
// ...      ,...
			if(temp_1<temp_minimum_price[count_2])
			{
				pnf[count_2].supp_price=temp_1;
				count_2++;
				break;
			}
// ...   -   Down
			else
			{
				pnf[count_2].supp_price=-10.0;
				trend='D';
				count_3=ArrayMaximum(temp_maximum_price,count_2-count_3,count_3);
				temp_2=count_3;
				temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
				while(count_3<count_2)
				{
					if(temp_1>temp_maximum_price[count_3])
					{
						count_3++;
						temp_1=NormalizeDouble(temp_1-double_box,symbol_digit);
					}
					else
					{
						count_3=ArrayMaximum(temp_maximum_price,count_2-count_3,count_3);
						temp_2=count_3;
						temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
					};
				};
				count_3=temp_2;
				temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
				pnf[count_3].resist_price=temp_1;
				count_3++;
				while(count_3<=count_2)
				{
					temp_1=NormalizeDouble(pnf[count_3-1].resist_price-double_box,symbol_digit);
					pnf[count_3].resist_price=temp_1;
					count_3++;
				}
				count_3=temp_2;
				count_2++;
				break;
			};
		};
//   - Down...
		case 'D':
		{
			temp_1=NormalizeDouble(pnf[count_2-1].resist_price-double_box,symbol_digit);
// ...      ,...
			if(temp_1>temp_maximum_price[count_2])
			{
				pnf[count_2].resist_price=temp_1;
				count_2++;
				break;
			}
// ...   -   Up
			else
			{
				pnf[count_2].resist_price=-10.0;
				trend='U';
				count_3=ArrayMinimum(temp_minimum_price,count_2-count_3,count_3);
				temp_2=count_3;
				temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
				while(count_3<count_2)
				{
					if(temp_1<temp_minimum_price[count_3])
					{
						count_3++;
						temp_1=NormalizeDouble(temp_1+double_box,symbol_digit);
					}
					else
					{
						count_3=ArrayMinimum(temp_minimum_price,count_2-count_3,count_3);
						temp_2=count_3;
						temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
					};
				};
				count_3=temp_2;
				temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
				pnf[count_3].supp_price=temp_1;
				count_3++;
				while(count_3<=count_2)
				{
					temp_1=NormalizeDouble(pnf[count_3-1].supp_price+double_box,symbol_digit);
					pnf[count_3].supp_price=temp_1;
					count_3++;
				}
				count_3=temp_2;
				count_2++;
				break;
			};
		};
	};
	return(0);
};

int UpdateTrend()
{
	switch(trend)
	{
//   - Up...
		case 'U':
		{
			temp_1=NormalizeDouble(pnf[column_count-2].supp_price+double_box,symbol_digit);
// ...      ,...
			if(temp_1<temp_minimum_price[column_count-1])
			{
				pnf[column_count-1].supp_price=temp_1;
				break;
			}
// ...   -   Down
			else
			{
				pnf[column_count-1].supp_price=-10.0;
				trend='D';
				count_3=ArrayMaximum(temp_maximum_price,count_2-count_3,count_3);
				temp_2=count_3;
				temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
				while(count_3<column_count-1)
				{
					if(temp_1>temp_maximum_price[count_3])
					{
						count_3++;
						temp_1=NormalizeDouble(temp_1-double_box,symbol_digit);
					}
					else
					{
						count_3=ArrayMaximum(temp_maximum_price,count_2-count_3,count_3);
						temp_2=count_3;
						temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
					};
				};
				count_3=temp_2;
				temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
				pnf[count_3].resist_price=temp_1;
				count_3++;
				while(count_3<=column_count-1)
				{
					temp_1=NormalizeDouble(pnf[count_3-1].resist_price-double_box,symbol_digit);
					pnf[count_3].resist_price=temp_1;
					count_3++;
				}
				count_3=temp_2;
				break;
			};
		};
//   - Down...
		case 'D':
		{
			temp_1=NormalizeDouble(pnf[column_count-2].resist_price-double_box,symbol_digit);
// ...      ,...
			if(temp_1>temp_maximum_price[column_count-1])
			{
				pnf[column_count-1].resist_price=temp_1;
				break;
			}
// ...   -   Up
			else
			{
				pnf[column_count-1].resist_price=-10.0;
				trend='U';
				count_3=ArrayMinimum(temp_minimum_price,count_2-count_3,count_3);
				temp_2=count_3;
				temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
				while(count_3<column_count-1)
				{
					if(temp_1<temp_minimum_price[count_3])
					{
						count_3++;
						temp_1=NormalizeDouble(temp_1+double_box,symbol_digit);
					}
					else
					{
						count_3=ArrayMinimum(temp_minimum_price,count_2-count_3,count_3);
						temp_2=count_3;
						temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
					};
				};
				count_3=temp_2;
				temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
				pnf[count_3].supp_price=temp_1;
				count_3++;
				while(count_3<=column_count-1)
				{
					temp_1=NormalizeDouble(pnf[count_3-1].supp_price+double_box,symbol_digit);
					pnf[count_3].supp_price=temp_1;
					count_3++;
				}
				count_3=temp_2;
				break;
			};
		};
	};
	return(0);
};